/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef flxvi_flxvi_H
#define flxvi_flxvi_H

#include "flxvi_titlebar.h"
#include "flxvilog.h"
#include "flxvi_flxinput.h"
#include "flxvi_cmdbar.h"


class FlxViManager : public FlxViManager_base {
  private:
    std::ostringstream& flxOutputStream;
    std::ostringstream& flxErrorStream;
    
    int NC_ROW, NC_COL;
    /**
    * @brief Windows
    */
    FlxViCmdBar *viCmdBar;			// Window for input of vi-Commands
    FlxViTitle* viTitle;			// Title-Bar
    FlxViLog* viLog;				// Log-Window
    FlxVi_FlxInput* viFlxInput;			// Fesslix-Input-Window
    FlxViDialogs viDlg;				// Some dialogs ....
    
  public:
    FlxViManager(std::ostringstream& flxOutputStream, std::ostringstream& flxErrorStream);
    ~FlxViManager();
    /**
    * @brief Updates all windows on the screen
    */
    void updateScreen();
    /**
    * @brief error: unknown command
    */
    void throw_CMDerror(const std::string &errIndex, const std::string &errmsg);
    /**
    * @brief deletes the currently displayed error message
    */
    void reset_error();
    /**
    * @brief Writes the output to the logging window
    */
    void do_log();
    
    /**
    * @brief starts the main part of the program
    */
    void execute();
    
    
    
    static std::string get_version();
};



#endif // flxvi_flxvi_H
