/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#include "flxvi_cmdbar.h"


FlxViCmdBar::FlxViCmdBar(int y, int x, int sx)
{
  create_win(win_cmdbar,y,x,1,sx,COLOR_PAIR(2),false);
  win_vi_cmd_clean();
}

FlxViCmdBar::~FlxViCmdBar()
{
  delwin(win_cmdbar);
}

void FlxViCmdBar::updateWindow(bool doupdateB)
{
  touchwin(win_cmdbar);
  wnoutrefresh(win_cmdbar);
  if (doupdateB) doupdate();
}

void FlxViCmdBar::win_vi_cmd_clean(int isstart, std::string txt, int strpos)
{
  int cp;
  if (isstart==0) cp = 2;
  else if (isstart==1) cp = 3;
  else if (isstart==2) cp = 4;
  if (isstart==1) curs_set(1);
  else curs_set(0);
  wattrset(win_cmdbar,COLOR_PAIR(cp));
//   box(win_cmdbar, ' ', ' ');
  wborder(win_cmdbar, ' ', ' ', ' ',' ',' ',' ',' ',' ');
  wmove(win_cmdbar,0,0);
  txt = ":" + txt;
  if (isstart==1) waddstr(win_cmdbar,txt.c_str());
  if (strpos>=0) wmove(win_cmdbar,0,strpos+1);
  wrefresh(win_cmdbar);
}

void FlxViCmdBar::vi_cmd_error(std::string errIndex, std::string errmsg)
{
  win_vi_cmd_clean(2);
  std::string es = "ERROR(" + errIndex + "): " + errmsg;
  waddstr(win_cmdbar,es.c_str());
  mvwchgat(win_cmdbar,0,0,5,A_BOLD,4,NULL);
  wrefresh(win_cmdbar);
}

void FlxViCmdBar::reset_error()
{
  win_vi_cmd_clean(0);
}

void FlxViCmdBar::input_loop(std::string& strinput, bool extentedCMD)
{
  bool cmdmode = false;		// false: only one character allowed; true: multiple characters allowed
  strinput = "";
  int strpos = 0;
  while (true) {
    int ch = getch();
    if ( ch==13 || ch==10 ) {	// command accomplished
      if (cmdmode) {
	win_vi_cmd_clean();
	mend_cmd(strinput);
	return;
      }
    } else if (ch==27) {		// delete the current command
      if (cmdmode) {
	cmdmode = false;
	win_vi_cmd_clean();
	strinput = "";
	strpos = 0;
      }
    } else if (extentedCMD && cmdmode==false && ch==58) {		// switch over to the extented command modus
      cmdmode = true;
      win_vi_cmd_clean(1);
    } else if ( ch==KEY_UP || ch==KEY_DOWN ) {
    } else if ( ch==KEY_LEFT || ch==KEY_RIGHT ) {
      if (cmdmode) {
	if (ch==KEY_LEFT) {
	  if (strpos>0) --strpos;
	} else {
	  if (strpos<(int)strinput.length()) ++strpos;
	}
	win_vi_cmd_clean(1,strinput,strpos);
      }
    } else if (cmdmode && ch==KEY_DC) {	// Entf-Key
      if (strpos<(int)strinput.length()) strinput.erase(strpos,1);
      win_vi_cmd_clean(1,strinput,strpos);
    } else if (cmdmode && ch==KEY_BACKSPACE) {	// Löschen-Taste
      if (strpos>0) strinput.erase(--strpos,1);
      win_vi_cmd_clean(1,strinput,strpos);
    } else if (cmdmode && (ch==KEY_HOME || ch==KEY_END) ) {
      if (ch==KEY_HOME) strpos=0;
      else strpos=strinput.length();
      win_vi_cmd_clean(1,strinput,strpos);
    } else {				// any other character
      char tc = ch;
      if (cmdmode) {
	strinput.insert(strpos,1,tc);
	  strpos++;
	win_vi_cmd_clean(1,strinput,strpos);
      } else {
	strinput.append(1,tc);
	return;
      }
    }
  } 
}
