/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#include "flxvi_flxinput.h"

FesslixMain* FlxViObj_Flx::fesslix_ptr = NULL;




cmdHistory::cmdHistory()
:is_parsing(false)
{

}

const std::string cmdHistory::parse_history(const bool getPrevious, const std::string lastEdit)
{
  if (histList.empty()) return lastEdit;
  if (!is_parsing) {
    if (getPrevious) {
      is_parsing = true;
      hist_it = histList.end();
      --hist_it;
      return (*hist_it);
    } else {
      return lastEdit;
    }
  } else {
    if (getPrevious) {
      if (hist_it!=histList.begin()) {
	--hist_it;
      }
      return (*hist_it);
    } else {
      ++hist_it;
      if (hist_it==histList.end()) {
	is_parsing = false;
	return lastEdit;
      } else {
	return (*hist_it);
      }
    }
  }
}

void cmdHistory::set_new_entry(const std::string lastEdit)
{
  is_parsing = false;
  if (lastEdit.empty()) return;
  if (histList.empty()==false) {
    hist_it = histList.end();
    --hist_it;
    if ( (*hist_it)==lastEdit ) return;
  }
  histList.push_back(lastEdit);
}




FlxVi_FlxInput::FlxVi_FlxInput(int y, int x, int sy, int sx, std::ostringstream& flxErrorStream)
:flxErrorStream(flxErrorStream),
strInput(""),strInput_prev(""),cursor_pos(0),is_activated(false)
{
  create_win(win_flxinput,y,x,sy,sx,COLOR_PAIR(1));
}

FlxVi_FlxInput::~FlxVi_FlxInput()
{
  delwin(win_flxinput);
}

void FlxVi_FlxInput::updateWindow(bool doupdateB)
{
  refresh_win_contents();
  touchwin(win_flxinput);
  wnoutrefresh(win_flxinput);
  if (doupdateB) doupdate();
}

const std::string FlxVi_FlxInput::activate()
{
  activateWindow(true);
  
  while (true) {
    bool has_been_edit = true;
    int ch = getch();

    if ( ch == 27 ) {			// ESC
//       strInput = strInput_prev;
      activateWindow(false);
      return "";
    } else if (ch==KEY_UP || ch==KEY_DOWN || ch==KEY_NPAGE || ch==KEY_PPAGE ) {
      has_been_edit = false;
      strInput = theHistory.parse_history(ch==KEY_UP||ch==KEY_PPAGE,strInput_prev);
      cursor_pos = strInput.length();
    } else if (ch==KEY_BACKSPACE) {	// Löschen-Taste
      if (cursor_pos>0) strInput.erase(--cursor_pos,1);
    } else if (ch==KEY_DC) {	// Entf-Key
      if (cursor_pos<strInput.length()) strInput.erase(cursor_pos,1);
    }  else if (ch==KEY_LEFT || ch==KEY_RIGHT ) {
      has_been_edit = false;
      if (ch==KEY_LEFT) {
	if (cursor_pos>0) --cursor_pos;
      } else if (ch==KEY_RIGHT) {
	if (cursor_pos<strInput.length()) ++cursor_pos;
      }
    } else if (ch==KEY_HOME || ch==KEY_END) {
      has_been_edit = false;
      if (ch==KEY_HOME) cursor_pos=0;
      else cursor_pos=strInput.length();
    } else if (ch==13 || ch==10) {	// ENTER
      mend_cmd(strInput);
      if (!strInput.empty()) {
	if (strInput[strInput.length()-1]!=';') {
	  strInput+=";";
	}
      }
      ReadStream *reader = new ReadStream(strInput, true);
      try {
      fesslix_ptr->evaluate(reader);
      } catch (FlxException_NeglectInInteractive& e) {
	GlobalVar.prelog_write();
	flxErrorStream << std::endl << e.what() << std::endl;
	GlobalVar.logLevel_strong_reset();
	if (GlobalVar.check_logNOTcout()) {
	  GlobalVar.slog(1) << std::endl << e.what() << std::endl;
	}
      } catch (FlxEndE const &e) {
	delete reader;
	GlobalVar.logLevel_strong_reset();
	GlobalVar.slog(4) << "end: 'end;' was called" << std::endl;
	viManager_ptr->do_log();
	activateWindow(false);
	strInput = "";
	return "q";
      } catch (FlxException& e) {
	delete reader;
	GlobalVar.prelog_write();
	flxErrorStream << std::endl << e.what() << std::endl;
	viManager_ptr->do_log();
	GlobalVar.logLevel_strong_reset();
	if (GlobalVar.check_logNOTcout()) {
	  GlobalVar.slog(1) << std::endl << e.what() << std::endl;
	}
	strInput = "";
	throw;
      }
      delete reader;
      viManager_ptr->do_log();
      theHistory.set_new_entry(strInput);
      strInput = "";
      cursor_pos = 0;
    } else {				// everything else
//       if (ch==194 && getch()==176 ) {
// 	strInput.insert(cursor_pos,"°");
// 	cursor_pos+=2;
//       } else {
	char tc = ch;
	strInput.insert(cursor_pos,1,tc);
	++cursor_pos;
//       }
    }
    if (has_been_edit) strInput_prev = strInput;
    refresh_win_contents(true);
  }
  
}

void FlxVi_FlxInput::update_cursorPos()
{
  wmove(win_flxinput,1,11+cursor_pos);
}

void FlxVi_FlxInput::activateWindow(bool activateW)
{
  if (is_activated!=activateW) {
    is_activated=activateW;
    if (is_activated) {
      curs_set(1);
    } else {
      curs_set(0);
    }
  }
  wborder(win_flxinput, ACS_VLINE, ACS_VLINE, ACS_HLINE, ACS_HLINE, ACS_LTEE, ACS_RTEE, ACS_LLCORNER, ACS_LRCORNER);
  refresh_win_contents();
  updateWindow(true);
}

void FlxVi_FlxInput::refresh_win_contents(bool doupdateB)
{
  if (is_activated) {
    wattron(win_flxinput,COLOR_PAIR(3));
  }
  clear_win(win_flxinput,true);
  mvwprintw(win_flxinput,1,1,"fesslix:> ");
  wprintw(win_flxinput,"%s",strInput.c_str());
  if (is_activated) {
    update_cursorPos();
    wattroff(win_flxinput,COLOR_PAIR(3));
  }
  if (doupdateB) {
    wrefresh(win_flxinput);
  }
}







