/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef flxvi_flxvi_flxinput_H
#define flxvi_flxvi_flxinput_H

#include "flxvidlgs.h"
#include "fesslix.h"



class cmdHistory {
  private:
    std::list<std::string> histList;
    std::list<std::string>::iterator hist_it;
    /**
    * @brief true, if the list is currently being prased
    */
    bool is_parsing;
  public:
    cmdHistory();
    const std::string parse_history(const bool getPrevious, const std::string lastEdit);
    void set_new_entry(const std::string lastEdit);
};



class FlxViObj_Flx : public FlxViObj_Base {
  protected:
    static FesslixMain* fesslix_ptr;
  public:
    static void set_fesslix_ptr(FesslixMain* fesslix_ptrV) { fesslix_ptr = fesslix_ptrV; }
};


class FlxVi_FlxInput : public FlxViObj_Flx {
  private:
    std::ostringstream& flxErrorStream;
    WINDOW* win_flxinput;
    
    /**
    * @brief stores the command-history
    */
    cmdHistory theHistory;
    /**
    * @brief the contents of the current log
    */
    std::string strInput;
    std::string strInput_prev;
    /**
    * @brief the current position of the cursor
    */
    tuint cursor_pos;
    /**
    * @brief true, if the window is currently activated
    */
    bool is_activated;
    
    /**
    * @brief Activates/deactivates this window
    */
    void activateWindow(bool activateW = true);
    /**
    * @brief refreshes the contents of the window
    */
    void refresh_win_contents(bool doupdateB=false);
    /**
    * @brief refreshes the contents of the window
    */
    void update_cursorPos();
  public:
    /**
    * @brief creates the Flx-Input-interface
    * @param y y-coordinate of the upper left corner of the window
    * @param x x-coordinate of the upper left corner of the window
    * @param sy height of the window
    * @param sx width of the window
    */
    FlxVi_FlxInput(int y, int x, int sy, int sx, std::ostringstream& flxErrorStream);
    ~FlxVi_FlxInput();
    
    const std::string activate();
    void updateWindow(bool doupdateB=false);
};


#endif // flxvi_flxvi_flxinput_H

