/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#include "flxvilog.h"


FlxViLog_List::FlxViLog_List()
:totalLineNumb(0)
{
  
}

tulong FlxViLog_List::count_lineNumb(const std::string& msg2log)
{
  tulong lines = 1;
  size_t found = 0;
  found = msg2log.find('\n',found);
  while (found!=std::string::npos) {
    ++lines;
    ++found;	// increase found so that we find something new
    found = msg2log.find('\n',found);
  };
  return lines;
}

void FlxViLog_List::add_logmsg(std::string msg2log)
{
  // get rid of the last newline
    mend_cmd(msg2log);
    if (msg2log!="") {
      if (msg2log.rfind('\n')==(msg2log.length()-1)) {
	msg2log.erase(msg2log.length()-1,msg2log.length());
      }
    }
  FlxViLog_ListEntry le;
  le.msg = msg2log;
  le.lineNumb_start = totalLineNumb+1;
  le.lineNumb_delta = count_lineNumb(msg2log);
  totalLineNumb+=le.lineNumb_delta;
  loglist.push_back(le);
}

const std::string FlxViLog_List::get_strBlock(std::size_t sstart, std::size_t send)
{
  // check boundaries
    if (loglist.empty()) return "";
    if (sstart<1) sstart = 1;
    if (send>totalLineNumb) send = totalLineNumb;
  // find the start
    std::list<FlxViLog_ListEntry>::iterator it, itp;
    for ( it=loglist.begin() ; it != loglist.end(); ++it ) {
      if ((*it).lineNumb_start>(int)sstart) {
	break;
      }
      itp = it;
    }
  // fix the beginning
    std::string rtstr = (*itp).msg;
    std::size_t t, t2;
    if ((int)sstart>(*itp).lineNumb_start) {
      t = (*itp).lineNumb_start;
      t2 = 0;
      while (t < sstart) {
	t2 = rtstr.find('\n',t2);
	++t2;
	++t;
      }
      rtstr.erase(0,t2);
    }
  // add the remaining lines (and maybe even more)
    while ((*itp).lineNumb_start+(*itp).lineNumb_delta-1<(int)send) {
      ++itp;
      rtstr += "\n" + (*itp).msg;
    }
  // fix the ending
    t = (*itp).lineNumb_start+(*itp).lineNumb_delta-1;
    if (t>send) {
      t2 = std::string::npos;
      while (t > send) {
	t2 = rtstr.rfind('\n',t2);
	--t2;
	--t;
      }
      rtstr.erase(t2+1,rtstr.length());
    }
  return rtstr;
}

void FlxViLog::createtxtwindow(int colorID)
{
  create_win(win_log_txt,txt_y,txt_x,txt_sy,txt_sx,COLOR_PAIR(colorID),false);
}

FlxViLog::FlxViLog(int y, int x, int sy, int sx)
:lw_size((sy>2)?(sy-2):1),scroll_x((sx>2)?(sx-2):1),pbegin(0),pend(0),
txt_y(y+1), txt_x(x+1), txt_sy(sy-2), txt_sx(sx-4)
{
  create_win(win_log,y,x,sy,sx,COLOR_PAIR(1));
  createtxtwindow(1);
}

FlxViLog::~FlxViLog()
{
  delwin(win_log_txt);
  delwin(win_log);
}

void FlxViLog::updateWindow(bool doupdateB)
{
  touchwin(win_log);
  touchwin(win_log_txt);
  wnoutrefresh(win_log);
  wnoutrefresh(win_log_txt);
  if (doupdateB) {
    doupdate();
  }
}

void FlxViLog::do_log(const std::string& msg2log)
{
  bool update_Log = false;
  if (pend==loglist.get_totalLineNumb()) {
    update_Log = true;
  }
  loglist.add_logmsg(msg2log);
  if (update_Log) {
    tulong pend_t = loglist.get_totalLineNumb();
    tulong pbegin_t;
    if (lw_size > pend_t) pbegin_t = 1;
    else pbegin_t = pend_t + 1 - lw_size;
    print_log(pbegin_t, pend_t);
  } else {
    print_log(pbegin,pend);	// to fix the scroll-bar
  }
}

void FlxViLog::print_log(tulong pbegin_t, tulong pend_t)
{
  // check boundaries
    if (pbegin==0 && pbegin_t>0) pbegin = 1;
    if (pbegin==0) {
      updateWindow(true);
      return;
    }
    if (pend_t>loglist.get_totalLineNumb()) pend_t = loglist.get_totalLineNumb();
    strCurLog = loglist.get_strBlock(pbegin_t,pend_t);
    pbegin = pbegin_t;
    pend = pend_t;
    strCurLog+="\n";
  // do the printing
    mvwprintw(win_log_txt,0,0,"%s",strCurLog.c_str());
  // draw the scroll-bar
    tdouble pv = (pend-pbegin+1.0)/loglist.get_totalLineNumb();
    tdouble ps = (pbegin-1.0)/loglist.get_totalLineNumb();
    tdouble pl = 1.0/lw_size;
    tdouble td = 0.0;
    bool started = false;
    
    for (tuint i=1;i<=lw_size;++i) {
      if (ps>=td && ps<=td+pl) {
	started = true;
	wattron(win_log,A_REVERSE);
      }
      mvwaddch(win_log,i,scroll_x,' ');
      if (ps+pv>=td && ps+pv<=td+pl) {
	started = false;
	wattroff(win_log,A_REVERSE);
      }
      td+=pl;
    }
    if (started) wattroff(win_log,A_REVERSE);
  // write some comments
    wborder(win_log, ACS_VLINE, ACS_VLINE, ACS_HLINE, ACS_HLINE, ACS_ULCORNER, ACS_URCORNER, ACS_LTEE, ACS_RTEE);
    mvwprintw(win_log,0,2," Lines [%i;%i] out of %i ",pbegin,pend,loglist.get_totalLineNumb());
  // refresh the windows
    updateWindow(true);
}

const std::string FlxViLog::activate()
{
  activateWindow(true);
  
  while (true) {
    int ch = getch();
    viManager_ptr->reset_error();
    if (ch == 'q' || ch == 27) {
      activateWindow(false);
      return "";
    } else if (ch==KEY_UP || ch==KEY_DOWN || ch==KEY_NPAGE || ch==KEY_PPAGE || ch==13 || ch==10 || ch==KEY_HOME || ch==KEY_END ) {
      tulong pbegin_t = pbegin;
      tulong pend_t = pend;
      if ( ch==KEY_UP ) {
	if (pbegin_t>1) {
	  --pbegin_t;
	  --pend_t;
	}
      } else if ( ch==KEY_DOWN || ch==13 || ch==10 ) {
	if (pend_t<loglist.get_totalLineNumb()) {
	  ++pbegin_t;
	  ++pend_t;
	}
      } else if ( ch==KEY_NPAGE ) {
	pend_t+=lw_size;
	if (pend_t>loglist.get_totalLineNumb() ) {
	  pend_t = loglist.get_totalLineNumb();
	} 
	if (pend_t<lw_size) {
	  pbegin_t = 1;
	} else {
	  pbegin_t = 1 + pend_t - lw_size;
	}
      } else if ( ch==KEY_PPAGE ) {
	if (pbegin_t>lw_size) {
	  pbegin_t-=lw_size;
	} else {
	  pbegin_t = 1;
	}
	pend_t = pbegin_t + lw_size - 1;
	if (pend_t>loglist.get_totalLineNumb()) {
	  pend_t = loglist.get_totalLineNumb();
	}
      } else if ( ch==KEY_HOME ) {
	pbegin_t = 1;
	pend_t = pbegin_t + lw_size - 1;
	if (pend_t>loglist.get_totalLineNumb()) {
	  pend_t = loglist.get_totalLineNumb();
	}
      } else if ( ch==KEY_END) {
	pend_t = loglist.get_totalLineNumb();
	if (pend_t<lw_size) {
	  pbegin_t = 1;
	} else {
	  pbegin_t = 1 + pend_t - lw_size;
	}
      }
      print_log(pbegin_t,pend_t);
    } else {
      const char chc[] = { char(ch), '\0' };
      std::string chs(chc);
      viManager_ptr->throw_CMDerror("FlxViLog::activate","'" + chs + "' is not a valid command");
    }
  }
 
}

void FlxViLog::activateWindow(bool activateW)
{
  int cp = activateW?3:1;
  delwin(win_log_txt);
  createtxtwindow(cp);
//   wattrset(win_log_txt,COLOR_PAIR(cp));
  win_replot_border(win_log);
  print_log(pbegin,pend);
}

