/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef flxvi_flxvilog_H
#define flxvi_flxvilog_H

#include "flxvidlgs.h"

#include <list>


struct FlxViLog_ListEntry {
  /**
  * @brief the line number of the first line of the message
  */
  int lineNumb_start;
  /**
  * @brief the length of the message in line numbers
  */
  int lineNumb_delta;
  /**
  * @brief the message itself
  */
  std::string msg;
};

class FlxViLog_List {
  private:
    tulong totalLineNumb;
    std::list<FlxViLog_ListEntry> loglist;
    
    tulong count_lineNumb(const std::string &msg2log);
  public:
    FlxViLog_List();
    void add_logmsg(std::string msg2log);
    const tulong get_totalLineNumb() { return totalLineNumb; }
    const std::string get_strBlock(std::size_t sstart, std::size_t send);
};

class FlxViLog : public FlxViObj_Base {
  private:
    WINDOW* win_log;
    WINDOW* win_log_txt;
    FlxViLog_List loglist;
    
    /**
    * @brief maximum number of lines in the logging window
    */
    tuint lw_size;
    /**
    * @brief x-coordinate of the scroll-bar
    */
    int scroll_x;
    /**
    * @brief the first line in the log to be printed
    */
    tulong pbegin;
    /**
    * @brief the last line in the log to be printed
    */
    tulong pend;
    /**
    * @brief the contents of the current log
    */
    std::string strCurLog;
    
    /**
    * @brief size-information of the txt-window
    */
    tuint txt_y, txt_x, txt_sy, txt_sx;
    
    /**
    * @brief Activates/deactivates this window
    */
    void activateWindow(bool activateW = true);
    void createtxtwindow(int colorID);
  public:
    /**
    * @brief creates the logging-interface
    * @param y y-coordinate of the upper left corner of the window
    * @param x x-coordinate of the upper left corner of the window
    * @param sy height of the window
    * @param sx width of the window
    */
    FlxViLog(int y, int x, int sy, int sx);
    ~FlxViLog();
    
    const std::string activate();
    void updateWindow(bool doupdateB=false);
    
    /**
    * @brief Logs the message msg2log
    */
    void do_log(const std::string& msg2log);
    /**
    * @brief sents the relevant part of the log to the output
    */
    void print_log(tulong pbegin_t, tulong pend_t);
};




#endif // flxvi_flxvilog_H

